<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

	<xsl:template match="grid">
		<xsl:variable name="formid" select="$formid" />
		<xsl:if test="not(@mode) and (not(contains(@lock,'true')) or not(@lock))">
			<xsl:variable name="columnwidth">
				<xsl:choose>
					<xsl:when test="@columnwidth != ''">
						<xsl:value-of select="@columnwidth" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="calc_columnwith" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<div>
				<xsl:if test="@id[.!='']"><xsl:attribute name="id"><xsl:value-of select="@id" /></xsl:attribute></xsl:if>
				<xsl:attribute name="class">block <xsl:value-of select="@class" /></xsl:attribute>
			
				<div class="header gridheader">
					<xsl:if test="@header!=''"><div class="gridtitle"><xsl:value-of select="@header" /></div></xsl:if>
					<xsl:if test="controls/*">
						<div class="controls gridcontrols">
							<div class="icons">
								<xsl:call-template name="controls" >
									<xsl:with-param name="controls" select="controls"/>
								</xsl:call-template>
							</div>
						</div>
					</xsl:if>
				</div>
				<div>
					<xsl:attribute name="class">
						<xsl:choose>
							<xsl:when test="@class">detail <xsl:value-of select="@class"/>
							</xsl:when>
							<xsl:otherwise>detail</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
					<xsl:choose>
						<xsl:when test="@legend[.!='']">
							<fieldset>
								<xsl:attribute name="class">
									<xsl:value-of select="@class" />
								</xsl:attribute>
								<legend>
									<xsl:value-of select="@legend" />
								</legend>
								<xsl:apply-templates>
									<xsl:with-param name="class">row</xsl:with-param>
									<xsl:with-param name="columnwidth">
										<xsl:value-of select="$columnwidth" />
									</xsl:with-param>
									<xsl:with-param name="firstcolumnwidth">
										<xsl:value-of select="@firstcolumnwidth" />
									</xsl:with-param>
								</xsl:apply-templates>
							</fieldset>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates>
								<xsl:with-param name="class">
									<xsl:value-of select="@mode" />
								</xsl:with-param>
								<xsl:with-param name="columnwidth">
									<xsl:value-of select="$columnwidth" />
								</xsl:with-param>
								<xsl:with-param name="firstcolumnwidth">
									<xsl:value-of select="@firstcolumnwidth" />
								</xsl:with-param>
							</xsl:apply-templates>
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
		</xsl:if>
	</xsl:template>

	<xsl:template match="layout">
		<!--<xsl:variable name="formid" select="$formid" />-->
		<!-- handled somewhere else -->
	</xsl:template>

	<xsl:template match="attrset">
		<xsl:param name="class" />
		<xsl:param name="columnwidth" />
		<xsl:param name="firstcolumnwidth" />
		<xsl:variable name="formid" select="$formid" />
		
		<!-- Attrset row -->
		<span>
			<xsl:if test="@class != '' or $class != ''">
				<xsl:attribute name="class">
					<xsl:choose>
						<xsl:when test="@class != ''">
							<xsl:value-of select="@class" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$class" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
			</xsl:if>
			<!-- Attr columns -->
			<xsl:for-each select="attr[@fieldtype!='hidden' and @fieldtype!='hidden2' and @fieldtype!='kpwindow' and not(@mode) and not(../@mode) and (not(@diag_mode) or (contains(@diag_mode,$DD_DIAG_MODE) and $DD_DIAG_MODE!='')) and (not(contains(@lock,'true')) or not(@lock))]">

				<xsl:if test="@label and @fieldtype!='label'">
					<!-- label -->
					<span>
						<xsl:attribute name="class">
							<xsl:text>column </xsl:text>
							<xsl:value-of select="@class" />
						</xsl:attribute>
						<xsl:if test="$columnwidth!=''">
							<xsl:choose>
								<xsl:when test="$firstcolumnwidth!='' and position()=1">
									<xsl:attribute name="style">width:<xsl:value-of select="$firstcolumnwidth"/>%</xsl:attribute>
								</xsl:when>
								<xsl:when test="@lwidth[.!='']">
									<xsl:attribute name="style">width:<xsl:value-of select="@lwidth"/>
									</xsl:attribute>
								</xsl:when>
								<xsl:when test="@width">
									<xsl:if test="@width != ''">
										<xsl:attribute name="style">width:<xsl:value-of select="@width"/>
										</xsl:attribute>
									</xsl:if>
								</xsl:when>
								<xsl:when test="@lspan[.!='' and .!='0']">
									<xsl:attribute name="style">width:<xsl:value-of select="$columnwidth*@lspan"/>%</xsl:attribute>
								</xsl:when>
								<xsl:otherwise>
									<xsl:attribute name="style">width:<xsl:value-of select="$columnwidth" />%</xsl:attribute>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
						<xsl:call-template name="disp_label" />
					</span>
				</xsl:if>

				<!-- field -->
				<span>
					<xsl:attribute name="class">
						<xsl:text>column </xsl:text>
						<xsl:value-of select="@class" />
					</xsl:attribute>
					<xsl:if test="$columnwidth!=''">
						<xsl:choose>
							<xsl:when test="$firstcolumnwidth!='' and position()=1">
								<xsl:attribute name="style">width:<xsl:value-of select="$firstcolumnwidth"/>%</xsl:attribute>
							</xsl:when>
							<xsl:when test="@width">
								<xsl:if test="@width != ''">
									<xsl:attribute name="style">width:<xsl:value-of select="@width"/>
									</xsl:attribute>
								</xsl:if>
							</xsl:when>
							<xsl:when test="@fspan[.!='' and .!='0']">
								<xsl:attribute name="style">width:<xsl:value-of select="$columnwidth*@fspan"/>%</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="style">width:<xsl:value-of select="$columnwidth" />%</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:if>

					<xsl:apply-templates select="." />
					<xsl:if test="@fieldtype!='submit_button' and @fieldtype!='reset_button' and @fieldtype!='onclick_button' and @fieldtype!='hard_url_button' and @fieldtype!='soft_url_button' and @fieldtype!='popup_win_button' and @fieldtype!='graph_submit_button' and @fieldtype!='graph_onclick_button' and @fieldtype!='confirm_button'">
						<span>
							<xsl:attribute name="id">errorsDiv<xsl:value-of select="generate-id(../../..)"/>_<xsl:value-of select="@id"/>
							</xsl:attribute>
							<xsl:comment>//</xsl:comment>
						</span>
					</xsl:if>
				</span>
			</xsl:for-each>
		</span>
	</xsl:template>

	<xsl:template name="calc_columnwith">
		<xsl:variable name="formid" select="$formid" />
		<xsl:for-each select="attrset">
			<xsl:sort select="count(attr[ (@label!='' and (@fieldtype!='label' or content)) and @fieldtype!='hidden' and @fieldtype!='hidden2' and (not(contains(@lock,'true')) or not(@lock))])*2 + count(attr[ (not(@label) or (@fieldtype='label' and not(content))) and @fieldtype!='hidden' and @fieldtype!='hidden2' and (not(contains(@lock,'true')) or not(@lock))])" order="descending"/>
			<xsl:if test="position() = 1">
				<!-- the -4 at the end is for IE cause 100% is content+scrollbar so 50%+50%+scrollbar > 100% and then the rows wrap :( then it is also needed for margin spaces
					counts attr with label for 2 collums, except for fieldtype label, counts attrs without label for 1
					and does not count fieldtype hidden and hidden2
				-->
				<!--<xsl:value-of select="floor(100 div (count(attr)*2))-2"/>-->
				<!--				<xsl:value-of select="floor(100 div (count(attr[ (@label!='' and (@fieldtype!='label' or ./content)) and @fieldtype!='hidden' and @fieldtype!='hidden2' and (not(contains(@lock,'true')) or not(@lock))])*2 + count(attr[ (not(@label) or (@fieldtype='label' and not(content))) and @fieldtype!='hidden' and @fieldtype!='hidden2' and (not(contains(@lock,'true')) or not(@lock))])))-4"/> -->
				<xsl:value-of select="floor(100 div (count(attr[ (@label!='' and (@fieldtype!='label' or ./content)) and @fieldtype!='hidden' and @fieldtype!='hidden2' and (not(contains(@lock,'true')) or not(@lock))])*2 + count(attr[ (not(@label) or (@fieldtype='label' and not(content))) and @fieldtype!='hidden' and @fieldtype!='hidden2' and (not(contains(@lock,'true')) or not(@lock))])))"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>

</xsl:stylesheet>