<?xml version="1.0" encoding="UTF-8"?>
<!--
#
# System:	Dynamic Dialogue (DD) System file
#
# File:			questionaire.xsl
#
# Title:		Render questionaires.
#
# Description:  Render questionaires with attr array.
# 		
# History:			Version	Person	Remark
# 03-11-2006		1.0		RuBr	Created this file

<!DOCTYPE xsl:stylesheet [
	<!ENTITY nbsp "<">
	<!ENTITY copy "<td>">
]>
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0" xmlns:xl="http://www.innovationinvestments.nl/layout" >
	<xsl:import href="questiontype/text.xsl"/>
	<xsl:import href="questiontype/password.xsl"/>
	<xsl:import href="questiontype/labelfield.xsl"/>
	<xsl:import href="questiontype/date_picker.xsl"/>
	<xsl:import href="questiontype/date_picker_group.xsl"/>
	<xsl:import href="questiontype/popup.xsl"/>
	<xsl:import href="questiontype/popup_json.xsl"/>
	<xsl:import href="questiontype/popup_json_suggest.xsl"/>
	<xsl:import href="questiontype/textarea.xsl"/>
	<xsl:import href="questiontype/radio.xsl"/>
	<xsl:import href="questiontype/checkbox.xsl"/>
	<xsl:import href="questiontype/group.xsl"/>
	<xsl:import href="questiontype/textdiff.xsl"/>
	<xsl:import href="questiontype/comment.xsl"/>
	<xsl:import href="questiontype/file.xsl"/>
	<xsl:import href="questiontype/hidden.xsl"/>
	<xsl:import href="questiontype/separator.xsl"/>
	<xsl:import href="questiontype/block.xsl"/>
	<xsl:import href="questiontype/editor.xsl"/>
	<xsl:import href="questiontype/toggle.xsl"/>
	<xsl:import href="../layout.xsl"/>
	<xsl:template match="attr[@fieldtype='questionnaire']">
		<xsl:variable name="layout" select="document(/diag/descendant::layout)" />
		<xsl:variable name="formid" select="$formid" />
		
		<xsl:if test="@instance_deleted = 'true'">
			<h3 class="questionnaireRemovedMsg"><xsl:value-of select="//diag/@name" /> verwijderd</h3>
		</xsl:if>
		
		<xsl:choose>
			<xsl:when test="$layout/xl:layout!=''">
				<xsl:apply-templates select="$layout/xl:layout">
					<xsl:with-param name="dataset" select="." />
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>

				<xsl:if test="not(@mode) and (not(contains(@lock,'true')) or not(@lock))">
					<!-- javascript library for questionnaire functions -->

					<table class="questionnaire" >
						<th colspan="3">
							<xsl:choose>
								<xsl:when test="@label !=''">
									<xsl:value-of select="@label"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:text/>
								</xsl:otherwise>
							</xsl:choose>
						</th>


						<xsl:for-each select="item[@fieldtype!='q_hidden']">
							<tr>
								<xsl:apply-templates select=".">
									<xsl:with-param name="form_id" select="../@id" />
								</xsl:apply-templates>	
								<!--<td class="questionanswer">
								<xsl:if test="@helpText!=''">
									<div class="info">
										<div class="icon">
											<xsl:attribute name="onmouseover">
												showInfo(event,'<xsl:value-of select="generate-id()"/>','<xsl:value-of select="@helpText" />');
											</xsl:attribute>
											<a>
												<img border="0">
													<xsl:attribute name="src">images/icons/help.png</xsl:attribute>
													<xsl:attribute name="alt">Help</xsl:attribute>
												</img>
											</a>
											<div class="tooltip">
												<p id="{generate-id()}">...</p>
											</div>
										</div>
									</div>
								</xsl:if>
							</td>-->
							</tr>
						</xsl:for-each>
						<xsl:if test="item[@fieldtype='q_hidden']">
							<tr>
								<td>
									<xsl:apply-templates select="item[@fieldtype='q_hidden']">
										<xsl:with-param name="form_id" select="@id" />
									</xsl:apply-templates>	
								</td>
							</tr>
						</xsl:if>
					</table>
					<xsl:if test="not(@readonly) or @readony!='true'">
						<input type="hidden" name="component_onload" >
							<xsl:attribute name="value">
								<xsl:text>if (helpBoxes.length &gt; 0) { toggleHelp(helpBoxes[0]); }</xsl:text>
							</xsl:attribute>
						</input>
					</xsl:if>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="questiontext">
		<xsl:param name="overrides"/>
		<xsl:if test="@label or ./label and (not($overrides) or not($overrides/@nolabel) or $overrides/@nolabel!='true')">
		<td>
				
				<xsl:if test="$overrides and $overrides/@styleCSS[.!='']">
						<xsl:attribute name="style">
							<xsl:value-of select=" $overrides/@styleCSS"/>
						</xsl:attribute>
				</xsl:if>
				<xsl:if test="$overrides and $overrides/@rowspan[.!='']">
						<xsl:attribute name="rowspan">
							<xsl:value-of select=" $overrides/@rowspan"/>
						</xsl:attribute>
				</xsl:if>
				
				<xsl:attribute name="class">
					<xsl:choose>
						<xsl:when test="@class[.!='']">
							<xsl:value-of select="@class"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>questiontext</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
				
				<xsl:if test="$overrides and $overrides/@tag[.!='']">
						<xsl:attribute name="class">
							<xsl:value-of select="$overrides/@tag"/>
						</xsl:attribute>
				</xsl:if>
				
				<xsl:choose>
					<xsl:when test="$overrides and $overrides/@lspan[.!='']">
						<xsl:attribute name="lspan">
							<xsl:value-of select="$overrides/@lspan" />
						</xsl:attribute>
					</xsl:when>
					<xsl:when test="@fspan[.!='']">
						<xsl:attribute name="colspan">
							<xsl:value-of select="@fspan" />
						</xsl:attribute>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>

				<xsl:if test="@lspan[.!='']">
					<xsl:attribute name="colspan">
						<xsl:value-of select="@lspan" />
					</xsl:attribute>
				</xsl:if>


				<!-- display label text -->
				<span>
					<xsl:choose>
						<xsl:when test="@fieldtype != 'q_comment' and @fieldtype != 'q_separator' and @fieldtype != 'q_group' and @fieldtype != 'q_labelfield' and @fieldtype != 'q_hidden' and  (@validate[.!=''] or @fieldtype[.='date_picker'])">
							<label>
								<xsl:attribute name="class">questiontextlabel <xsl:if test="@required[.='true']">questiontextlabelrequired</xsl:if>
								</xsl:attribute>
								
								<xsl:attribute name="for">
									<xsl:value-of select="@id"/>
								</xsl:attribute>
								<xsl:choose>
									<xsl:when test="./label">
										<xsl:value-of select="./label" disable-output-escaping="yes" />
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="./@label" disable-output-escaping="yes"/>
									</xsl:otherwise>
								</xsl:choose>
							</label>
						</xsl:when>
						<xsl:otherwise>
							<label>
								<xsl:attribute name="class">questiontextlabel <xsl:if test="@required[.='true']">questiontextlabelrequired</xsl:if>
								</xsl:attribute>
								<xsl:choose>
									<xsl:when test="./label">
										<xsl:value-of select="./label" disable-output-escaping="yes" />
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="./@label" disable-output-escaping="yes"/>
									</xsl:otherwise>
								</xsl:choose>

							</label>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:if test="@helpText[.!='']">
						<a class="fa fa-info-circle floatright icon">
							<xsl:attribute name="id">
								<xsl:value-of select="generate-id()" />
							</xsl:attribute>
							<xsl:attribute name="onmouseover">displayTooltipContent('<xsl:value-of select="generate-id(.)" />', '<xsl:value-of select="@helpText"/>');</xsl:attribute>
							<xsl:comment>spacer</xsl:comment>			
						</a>
					</xsl:if>
				</span>
			</td>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>